package com.jz.dm.common.util;

import com.alibaba.fastjson.JSON;
import com.jz.dm.common.constant.LoggingConstants;
import org.apache.commons.lang3.StringUtils;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.*;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.common.util
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: MapUtil
 * @DATE: 2021-1-2/14:02
 * @DAY_NAME_SHORT: 周六
 * @Description:
 **/
public class MapUtil {


    /**
     * 获取签名参数
     * @param apiKey
     * @param method
     * @param signType
     * @return
     */
    public static String getSignValue(String apiKey,String method,String signType){
        StringBuilder builder = new StringBuilder();
        builder.append("apiKey=").append(apiKey).append(LoggingConstants.AND_SPILT)
               .append("method=").append(method).append(LoggingConstants.AND_SPILT)
               .append("signType=").append(signType);
        return builder.toString();
    }


    /**
     * 给map集合中的key实现字典排序
     * @param map
     * @return
     */
    public Map<String , Object> MapKeySort(Map<String , Object> map){
        ArrayList list = new ArrayList();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            list.add(entry.getKey());
            //System.out.println("key= " + entry.getKey() + " and value= " + entry.getValue());
        }

        HashMap<String , Object> newMap = new HashMap<String , Object>();
        //运用Collections的sort（）方法对其进行排序 sort（）方法需要传 连个参数，一个是需要进行排序的Collection 另一个是一个Comparator
        Collections.sort(list, new SpellComparatorUtils());
        for (int i = 0; i < list.size(); i++) {
            newMap.put(list.get(i).toString() , map.get(list.get(i).toString()));
        }
        return newMap;
    }

    /**
     * 将对象转成TreeMap,属性名为key,属性值为value
     *
     * @param object 对象
     * @return
     * @throws IllegalAccessException
     */
    public static TreeMap<String, String> objToMap(Object object) throws IllegalAccessException {

        Class clazz = object.getClass();
        TreeMap<String, String> treeMap = new TreeMap<String, String>();

        while (null != clazz.getSuperclass()) {
            Field[] declaredFields1 = clazz.getDeclaredFields();

            for (Field field : declaredFields1) {
                String name = field.getName();

                // 获取原来的访问控制权限
                boolean accessFlag = field.isAccessible();
                // 修改访问控制权限
                field.setAccessible(true);
                Object value = field.get(object);
                // 恢复访问控制权限
                field.setAccessible(accessFlag);

                if (null != value && StringUtils.isNotBlank(value.toString())) {
                    //如果是List,将List转换为json字符串
                    if (value instanceof List) {
                        value = JSON.toJSONString(value);
                    }
                    treeMap.put(name, String.valueOf(value));
                }
            }

            clazz = clazz.getSuperclass();
        }
        return treeMap;
    }

    /**
     * 按照指定的分割符将list转换为String
     *
     * @param list
     * @param separator
     * @return
     */
    public static String listToString(List list, String separator) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            if (i == list.size() - 1) {
                sb.append(list.get(i));
            } else {
                sb.append(list.get(i));
                sb.append(separator);
            }
        }
        return sb.toString();
    }

    /**
     * 　　* 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     * 　　* @param params 需要排序并参与字符拼接的参数组
     * 　　* @return 拼接后字符串
     * 　　* @throws UnsupportedEncodingException
     */
    public static String createLinkStringByGet(Map<String, String> params)  {
        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);
        String prestr = "";
        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value =  params.get(key);
            try {
                value = URLEncoder.encode(value, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            if (i == keys.size() - 1) {//拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }
        return prestr;
    }
class SpellComparatorUtils implements Comparator {
        @Override
        public int compare(Object o1, Object o2) {
            try {
                // 取得比较对象的汉字编码，并将其转换成字符串
                String s1 = new String(o1.toString().getBytes("GB2312"), "ISO-8859-1");
                String s2 = new String(o2.toString().getBytes("GB2312"), "ISO-8859-1");
                // 运用String类的 compareTo（）方法对两对象进行比较
                return s1.compareTo(s2);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return 0;
        }
    }

    public static void main(String[] args)  {
        Map<String, String> map = new HashMap();
        map.put("name", "hello");
        map.put("value", "world");
        System.out.println(createLinkStringByGet(map));
    }
}
