package com.jz.dm.config;

import com.alibaba.csp.sentinel.annotation.aspectj.SentinelResourceAspect;
import com.alibaba.csp.sentinel.slots.block.RuleConstant;
import com.alibaba.csp.sentinel.slots.block.degrade.DegradeRule;
import com.alibaba.csp.sentinel.slots.block.degrade.DegradeRuleManager;
import com.alibaba.csp.sentinel.slots.block.flow.FlowRule;
import com.alibaba.csp.sentinel.slots.block.flow.FlowRuleManager;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.config
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: SentinelConfig
 * @DATE: 2021-1-7/15:57
 * @DAY_NAME_SHORT: 周四
 * @Description: 限流熔断初始化配置
 **/
@Configuration
public class SentinelConfig {
    @Bean
    public SentinelResourceAspect sentinelResourceAspect() {
        return new SentinelResourceAspect();
    }

    /**
     * 默认最大请求数
     */
    @Value("${sentinel.request.max}")
    private Integer defaultMaxRequest;
    /**
     * 限流/熔断资源配置
     */
    @Value("${sentinel.resource}")
    private String limitSource ;

    @PostConstruct
    private void initFlowRules() {
        List<FlowRule> rules = new ArrayList<>();

        FlowRule rule = new FlowRule();
        rule.setResource(limitSource);
        rule.setGrade(RuleConstant.FLOW_GRADE_QPS);
        rule.setCount(defaultMaxRequest); //每秒中可以有多少个请求
        rules.add(rule);
        FlowRuleManager.loadRules(rules);

        //降级规则，可以多个degradeRule rule
        //DegradeRuleManager.getRules()可以获取到已经设置的降级规则
        List<DegradeRule> degradeRules = new ArrayList<>();
        DegradeRule degradeRule = new DegradeRule();
        //设置资源名称，sentinel降级都是以资源为单位进行
        degradeRule.setResource(limitSource);
        //使用异常统计降级,分钟统计,滑动时间窗口
        degradeRule.setGrade(RuleConstant.DEGRADE_GRADE_EXCEPTION_COUNT);
        //异常数达到的数量阈值
        degradeRule.setCount(defaultMaxRequest);
        //秒级时间窗口,该值必须有且必须大于零，否则降级将无法生效
        degradeRule.setTimeWindow(1);
        degradeRules.add(degradeRule);
        //重新加载限流规则，此处将覆盖原有的限流，所以如果想要不覆盖
        //请使用DegradeRuleManager.getRules()获取到的加入到rules中
        DegradeRuleManager.loadRules(degradeRules);
    }
}
