package com.jz.dm.filter;

import com.alibaba.fastjson.JSONObject;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.enums.Format;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.enums.SignType;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.StringUtil;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * 参数检查过滤器
 *
 * @author key
 */
@Component
public class CheckArgsFilter extends AbstractFilter {

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_2;
    }

    @Override
    public String getFilterName() {
        return "CheckArgsFilter";
    }

    @Value("${apiRequest.testExample.apiKey}")
    private String testApiKey;

    @Value("${apiRequest.testExample.sign}")
    private String testSign;

    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        if (null != request && StringUtil.isNotBlank(request.getParams())) {
            JSONObject params = JSONObject.parseObject(request.getParams());
            Boolean aBoolean = params.getBoolean("isTest");
            if (null == aBoolean){
                throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT); //无效参数
            }
            if (aBoolean) {//如果是测试数据
                chain.doFilter(assembleTestData(), response);
            } else {
                // 校验参数非空
                if (StringUtil.isEmpty(request.getApiKey()) || StringUtil.isEmpty(request.getMethod())
                        || StringUtil.isEmpty(request.getSignType()) || StringUtil.isEmpty(request.getSign())
                        || StringUtil.isEmpty(request.getTimestamp()) || StringUtil.isEmpty(request.getParams())) {
                    throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT); //无效参数
                }
                // 设置默认值
                if (StringUtil.isEmpty(request.getFormat())) {
                    request.setFormat(Format.JSON.name());
                }
                try {
                    request.setVersion("v1.0.0");
                    //格式，目前仅支持JSON
                    Format.valueOf(request.getFormat());
                    //请求使用的编码格式，如UTF-8,GBK,GB2312等
                    //Charset.forName(request.getCharset());
                    //生成签名字符串所使用的签名算法类型
                    SignType.valueOf(request.getSignType());
                } catch (Exception ex) {
                    //无效参数
                    throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT);
                }
            }
            chain.doFilter(request, response);
        }
    }

    /**
     * 组装测试数据
     */
    private GatewayRequest assembleTestData() {
        GatewayRequest gatewayRequest = new GatewayRequest();
        gatewayRequest.setApiKey(testApiKey);
        gatewayRequest.setMethod("query");
        gatewayRequest.setFormat(Format.JSON.name());
        gatewayRequest.setSignType("MD5");
        gatewayRequest.setSign(testSign);
        gatewayRequest.setTimestamp(System.currentTimeMillis() + "");
        gatewayRequest.setVersion("v1.0.0");
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("authCode","202100000001118191258T718d78591J");
        jsonObject.put("isTest",true);
        gatewayRequest.setParams(jsonObject.toString());
        return gatewayRequest;
    }

}
